@@ -25,10 +25,13 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'CFLAGS': '-fno-strict-aliasing -O2 -pipe -DNDEBUG -O2 -pipe   -fPIC',
  'CFLAGSFORSHARED': '-fPIC',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
- 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' '--srcdir=/usr/obj/ports/Python-2.7.11/Python-2.7.11' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-cxx_main' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
+ 'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' '--srcdir=/usr/obj/ports/Python-2.7.12/Python-2.7.12' '--with-fpectl' '--with-threads' '--enable-ipv6' '--with-cxx_main' '--with-system-expat' '--with-system-ffi' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python2.7',
- 'COREPYTHONPATH': ':plat-openbsd5:lib-tk:lib-old',
+ 'COREPYTHONPATH': ':plat-openbsd6:lib-tk:lib-old',
+ 'COVERAGE_INFO': '/usr/obj/ports/Python-2.7.12/Python-2.7.12/coverage.info',
+ 'COVERAGE_REPORT': '/usr/obj/ports/Python-2.7.12/Python-2.7.12/lcov-report',
+ 'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov report"',
  'CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
  'CXX': 'c++ -pthread',
  'C_THREADS': 0,
@@ -43,8 +46,8 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
  'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
+ 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
+ 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
  'DYNLOADFILE': 'dynload_shlib.o',
  'ENABLE_IPV6': 1,
  'ENSUREPIP': 'no',
@@ -136,7 +139,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_FTRUNCATE': 1,
  'HAVE_GAI_STRERROR': 1,
  'HAVE_GAMMA': 1,
- 'HAVE_GCC_ASM_FOR_X87': 1,
+ 'HAVE_GCC_ASM_FOR_X87': 0,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETCWD': 1,
  'HAVE_GETC_UNLOCKED': 1,
@@ -238,6 +241,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_RL_COMPLETION_MATCHES': 1,
  'HAVE_RL_COMPLETION_SUPPRESS_APPEND': 1,
  'HAVE_RL_PRE_INPUT_HOOK': 1,
+ 'HAVE_RL_RESIZE_TERMINAL': 1,
  'HAVE_ROUND': 1,
  'HAVE_SELECT': 1,
  'HAVE_SEM_GETVALUE': 1,
@@ -288,7 +292,6 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HAVE_STRUCT_STAT_ST_GEN': 1,
  'HAVE_STRUCT_STAT_ST_RDEV': 1,
  'HAVE_STRUCT_TM_TM_ZONE': 1,
- 'HAVE_ST_BLOCKS': 1,
  'HAVE_SYMLINK': 1,
  'HAVE_SYSCONF': 1,
  'HAVE_SYSEXITS_H': 1,
@@ -354,16 +357,16 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'HGBRANCH': '',
  'HGTAG': '',
  'HGVERSION': '',
- 'HOST_GNU_TYPE': 'i386-unknown-openbsd5.9',
+ 'HOST_GNU_TYPE': 'hppa-unknown-openbsd6.0',
  'HURD_C_THREADS': 0,
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include /usr/local/include/python2.7 /usr/local/include/python2.7',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python2.7',
- 'INSTALL': '/usr/obj/ports/Python-2.7.11/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-2.7.11/bin/install -c  -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-2.7.11/bin/install -c -s  -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-2.7.11/bin/install -c  -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-2.7.11/bin/install -c -m 555',
+ 'INSTALL': '/usr/obj/ports/Python-2.7.12/bin/install -c',
+ 'INSTALL_DATA': '/usr/obj/ports/Python-2.7.12/bin/install -c  -m 644',
+ 'INSTALL_PROGRAM': '/usr/obj/ports/Python-2.7.12/bin/install -c -s  -m 755',
+ 'INSTALL_SCRIPT': '/usr/obj/ports/Python-2.7.12/bin/install -c  -m 755',
+ 'INSTALL_SHARED': '/usr/obj/ports/Python-2.7.12/bin/install -c -m 555',
  'INSTSONAME': 'libpython2.7.so.0.0',
  'LDCXXSHARED': 'c++ -pthread -shared -fPIC',
  'LDFLAGS': '-L/usr/local/lib/',
@@ -392,9 +395,9 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'LLVM_PROF_MERGER': '',
  'LN': 'ln',
  'LOCALMODLIBS': '',
- 'MACHDEP': 'openbsd5',
- 'MACHDEPPATH': ':plat-openbsd5',
- 'MACHDEPS': 'plat-openbsd5',
+ 'MACHDEP': 'openbsd6',
+ 'MACHDEPPATH': ':plat-openbsd6',
+ 'MACHDEPS': 'plat-openbsd6',
  'MACHDEP_OBJS': '',
  'MACHDESTLIB': '/usr/local/lib/python2.7',
  'MACH_C_THREADS': 0,
@@ -433,12 +436,12 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'PGO_PROF_GEN_FLAG': '',
  'PGO_PROF_USE_FLAG': '',
  'PGSRCS': '\\',
- 'PLATDIR': 'plat-openbsd5',
+ 'PLATDIR': 'plat-openbsd6',
  'PLATMACDIRS': 'plat-mac plat-mac/Carbon plat-mac/lib-scriptpackages \\',
  'PLATMACPATH': ':plat-mac:plat-mac/lib-scriptpackages',
  'POBJS': '\\',
  'POSIX_SEMAPHORES_NOT_ENABLED': 0,
- 'PROFILE_TASK': '-m test.regrtest --pgo',
+ 'PROFILE_TASK': '-m test.regrtest --pgo -x test_asyncore test_gdb test_multiprocessing test_subprocess',
  'PSRCS': '\\',
  'PTHREAD_SYSTEM_SCHED_SUPPORTED': 1,
  'PURIFY': '',
@@ -448,7 +451,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'PYTHONFRAMEWORKDIR': 'no-framework',
  'PYTHONFRAMEWORKINSTALLDIR': '',
  'PYTHONFRAMEWORKPREFIX': '',
- 'PYTHONPATH': ':plat-openbsd5:lib-tk:lib-old',
+ 'PYTHONPATH': ':plat-openbsd6:lib-tk:lib-old',
  'PYTHON_FOR_BUILD': './python -E',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
@@ -464,7 +467,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'RANLIB': 'ranlib',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-2.7.11/Python-2.7.11',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-2.7.12/Python-2.7.12',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SGI_ABI': '',
@@ -479,7 +482,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'SIZEOF_FPOS_T': 8,
  'SIZEOF_INT': 4,
  'SIZEOF_LONG': 4,
- 'SIZEOF_LONG_DOUBLE': 12,
+ 'SIZEOF_LONG_DOUBLE': 8,
  'SIZEOF_LONG_LONG': 8,
  'SIZEOF_OFF_T': 8,
  'SIZEOF_PID_T': 4,
@@ -502,13 +505,13 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'SVNVERSION': 'echo Unversioned directory',
  'SYSLIBS': '-lm',
  'SYS_SELECT_WITH_SYS_TIME': 1,
- 'TANH_PRESERVES_ZERO_SIGN': 0,
+ 'TANH_PRESERVES_ZERO_SIGN': 1,
  'TCLTK_INCLUDES': '',
  'TCLTK_LIBS': '',
  'TESTOPTS': '-l',
  'TESTPATH': '',
  'TESTPROG': './Lib/test/regrtest.py',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-2.7.11/Python-2.7.11 ./python -Wd -3 -E -tt',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-2.7.12/Python-2.7.12 ./python -Wd -3 -E -tt',
  'TESTPYTHONOPTS': '',
  'THREADOBJ': 'Python/thread.o',
  'TIME_WITH_SYS_TIME': 1,
@@ -531,13 +534,14 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'WITH_THREAD': 1,
  'WITH_TSC': 0,
  'WITH_VALGRIND': 0,
- 'X87_DOUBLE_ROUNDING': 1,
+ 'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-2.7.11/Python-2.7.11',
- 'abs_srcdir': '/usr/obj/ports/Python-2.7.11/Python-2.7.11',
- 'build': 'i386-unknown-openbsd5.9',
+ 'abs_builddir': '/usr/obj/ports/Python-2.7.12/Python-2.7.12',
+ 'abs_srcdir': '/usr/obj/ports/Python-2.7.12/Python-2.7.12',
+ 'build': 'hppa-unknown-openbsd6.0',
+ 'cross_compiling': 'no',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
- 'host': 'i386-unknown-openbsd5.9',
+ 'host': 'hppa-unknown-openbsd6.0',
  'prefix': '/usr/local',
  'srcdir': '.'}
